import hydra_zen

import aprl.part
import aprl.utils

store = hydra_zen.ZenStore(overwrite_ok=True)
store(dict(), name="__placeholder", group="fn")


def add_fn(name: str, cfg):
    store(cfg, name=name, group="aprl/part/fn", package="fn")
    store.add_to_hydra_store(overwrite_ok=True)


aprl_run, aprl_cfg = aprl.part.register(aprl.utils.call)
aprl_pcfg = aprl.utils.make_partial(aprl_cfg)

if __name__ == "__main__":
    aprl_run()



from typing import Callable

import hydra
import hydra_zen

store = hydra_zen.ZenStore(overwrite_ok=True)
overrides_store = store(group="aprl/overrides", package="_global_")
store(dict(), name="__placeholder", group="overrides", package="_global_")
overrides_store(dict(), name="none")
store.add_to_hydra_store(overwrite_ok=True)

parts_store = store(group="aprl", package="_global_")

store(
    dict(), name="__placeholder", group="args"
)  # see https://github.com/facebookresearch/hydra/issues/2875
args_store = store(group="aprl/args", package="_global_")


# Store the config
def register(
    fn: Callable,
    base_args: dict | None = None,
    help_msg: str | None = None,
    default_sweep: dict | None = None,
    name: str | None = None,
    builds_kws: dict | None = None,
    zen_kws: dict | None = None,
) -> tuple[Callable, type]:
    """
    Generate hydra config corresponding to fn
    Register basic config with base_args values in group /aprl/part with name <name>
    Generate hydra endpoint for configurable CLI
    returns hydra endpoint and Dataclass to instantiate endpoint configurations

    Args:
        fn: Callable to be aprled
        base_args: Default configuration values, keys of the dictionnary should be arguments of fn
        help_msg: Help message to display when running CLI with --help. Default to fn.__doc__
        default_sweep: Configure default multirun configuration to launch with -m flag
        name: Config name in the store, default to fn.__name__
        builds_kws: Keyword arguments to be passed to hydra_zen.builds(fn, populate_full_signature=True, **kw),
        zen_kws: Keyword arguments to be passed to hydra_zen.builds(fn, **kw),

    Returns: hydra wrapped endpoint and Config dataclass

    """
    builds_kws = builds_kws or dict()
    zen_kws = zen_kws or dict()
    if name is None:
        name = fn.__name__
    if help_msg is None:
        help_msg = f"""

         {fn.__name__=} function wrapped in hydra 
        """ + (fn.__doc__ or "")

    base_config = hydra_zen.builds(
        fn,
        **(dict(populate_full_signature=True, zen_partial=False) | builds_kws),
    )

    if base_args is None:
        base_args = dict()

    base_args = hydra_zen.make_config(
        **base_args,
        bases=(base_config,),
        zen_dataclass={
            "cls_name": f'{"".join(x.capitalize() for x in name.lower().split("_"))}AprlPartArgs'
        },
    )

    args_store(base_args, name=name)

    _recipe = dict(
        hydra=dict(
            sweeper=dict(params=default_sweep),
            help=dict(header=help_msg, app_name=name),
        ),
        defaults=[
            {"args": name},
            {"overrides": "none"},
            "_self_",
        ],
    )

    parts_store(_recipe, name=name)
    store.add_to_hydra_store(overwrite_ok=True)
    overrides_store.add_to_hydra_store(overwrite_ok=True)

    # Create CLI endpoint
    api_endpoint = hydra.main(
        config_name="aprl/" + name, version_base="1.3", config_path="."
    )(hydra_zen.zen(fn, **zen_kws))

    return api_endpoint, base_args



import logging
from typing import Callable

import hydra
import hydra_zen

log = logging.getLogger(__name__)

store = hydra_zen.ZenStore(overwrite_ok=True)
params_store = store(group="aprl/params", package="params")
overrides_store = store(group="aprl/overrides", package="_global_")
store(dict(), name="__placeholder", group="params", package="_global_")
store(dict(), name="__placeholder", group="overrides", package="_global_")
overrides_store(dict(), name="none")


def run(
    to_run: list[str] = [],
    parts: dict[str, Callable] = dict(),
    params: dict = dict(),
    dry: bool = False,
):
    """

    Args:
        to_run: list of parts to run
        parts: dictionary of part function
        params: Place holder for exposing values that can be used in part configurations
        dry: if true does not execute the parts (can be useful for dispalying logs and debugging)
    """
    log.info("Starting")
    for part in to_run:
        log.info(f"Running part {part}")

        if hasattr(parts[part], "func"):
            log.debug(parts[part].func.__name__)
            log.debug(parts[part].func.__doc__)

        elif hasattr(parts[part], "__doc__"):
            log.debug(parts[part].__doc__)

        if not dry:
            parts[part]()
            log.info(f"part {part} done")

    log.info("Done")


# Wrap the function to accept the configuration as input
zen_endpoint = hydra_zen.zen(run)


# Store the config
def register(
    name: str, parts: dict, params: dict, help_msg="", default_sweep: dict = None
):
    """
    Register a aprl appareil config and return appareil hydra entrypoint and
    the Config recipe to generate new configuration

    Args:
        help_msg (): Message describing the parameter values
        name: Name of the appareil used for declaring the Config
        parts: mapping of part name to part function to be called
        params: dictionnary of the appareil parameters, declared in separate config group for easy extension
        default_sweep: default multirun configuration

    Returns:  appareil hydra main and appareil config recipe

    """
    if isinstance(params, dict):
        params = hydra_zen.make_config(**params)

    params_store(params, name=name)
    store = hydra_zen.store()

    base_config = hydra_zen.builds(
        run,
        populate_full_signature=True,
        zen_partial=True,
    )

    help_msg = (
        "    =============="
        + ""
        + help_msg
        + ""
        + """
Parts
\t"""
        + "\n\t".join([f"{s} -> {parts[s]._target_}" for s in sorted(parts)])
        + ""
        + f"""

Run  with "dry=True hydra.verbose=aprl.appareil" for displaying the doc of each part

Use `to_run=[<part>,...]` to run specify parts\n\n"""
        + ""
        + ""
    )

    _recipe = hydra_zen.make_config(
        to_run=tuple(sorted(parts)),
        parts=parts,
        bases=(base_config,),
        hydra=dict(
            sweeper=dict(params=default_sweep),
            help=dict(header=help_msg, app_name=name),
        ),
        hydra_defaults=[
            {"params": name},
            {"overrides": "none"},
            "_self_",
        ],
    )
    store(
        _recipe,
        name=name,
        package="_global_",
        group="aprl",
    )
    # Create a  partial configuration associated with the above function (for easy extensibility)

    store.add_to_hydra_store(overwrite_ok=True)
    overrides_store.add_to_hydra_store(overwrite_ok=True)
    params_store.add_to_hydra_store(overwrite_ok=True)

    with hydra.initialize(version_base="1.3", config_path="."):
        cfg = hydra.compose(
            "aprl/" + name,
        )

    recipe = hydra_zen.make_config(
        **{
            k: node
            for k, node in cfg.items()
            if k not in ("_target_", "_partial_", "_args_", "_convert_", "_recursive_")
        },
        bases=(base_config,),
        zen_dataclass={
            "cls_name": f'{"".join(x.capitalize() for x in name.lower().split("_"))}Recipe'
        },
    )
    # Create CLI endpoint
    api_endpoint = hydra.main(
        config_name="aprl/" + name, version_base="1.3", config_path="."
    )(zen_endpoint)

    return api_endpoint, recipe, params



import logging
from typing import Any, Callable

import toolz
from hydra_zen import instantiate, just
from omegaconf import OmegaConf

log = logging.getLogger(__name__)


def call(f):
    return f()


def to_list(d):
    l = [d[k] for k in sorted(d)]
    log.debug(f"Converted {d} to {l}")
    return l


KWARGS_PREFIX = "kwargs"
ARGS_PREFIX = "args"
STORE_PREFIX = "store"

PREFIXES = [STORE_PREFIX, KWARGS_PREFIX, ARGS_PREFIX]


def depinject(
    fn: Callable,
    flat_factories: dict[str, Callable],
    convert_to_list: None | list[str] = None,
    params: None | dict[str, Any] = None,
    paths: None | dict[str, str] = None,
):
    """
    Args:
        fn: callable to be injected with dependencies
        flat_factories: dictionary of callables that return the dependencies
        convert_to_list: list of path in the nested factory dict to convert to list
        params: placehoder argument for putting highlevel parameters for extension
        paths: placeholder argument for putting paths values
    """
    wrong_keys = [
        k for k in flat_factories if not any(k.startswith(p) for p in PREFIXES)
    ]
    if wrong_keys:
        log.warn(
            f"all factories keys should start with one of {PREFIXES}, {wrong_keys} will be ignored"
        )

    # From flat dot notation to nested dict of factories
    log.debug("Creating nested config of factories")
    factoried = dict()
    for k in sorted(flat_factories.keys(), reverse=False):
        kl = k.split(".")
        log.debug(f"Inserting {k} as {kl}")
        node_cfg = OmegaConf.create(just(flat_factories[k])())
        node_dict = OmegaConf.to_container(node_cfg)
        factoried = toolz.assoc_in(factoried, kl, node_dict)
    log.debug(f"Done : nested factory configs {factoried}")

    convert_to_list = convert_to_list or []
    log.debug(f"Will convert {convert_to_list} to list")
    updates_keys = sorted(list(flat_factories.keys()) + convert_to_list, reverse=True)
    for k in updates_keys:
        kl = k.split(".")
        if k in convert_to_list:
            log.debug(f"Converting {k} to list")
            factoried = toolz.update_in(factoried, kl, to_list)
        else:
            log.debug(f"Instantiating and calling {k}")
            factoried = toolz.update_in(
                factoried, kl, toolz.compose_left(instantiate, call)
            )

    # convert dict node to value list  + call each factory
    kwargs = factoried.get(KWARGS_PREFIX, dict())
    args = to_list(factoried.get(ARGS_PREFIX, {}))
    store = factoried.get(STORE_PREFIX, lambda x: x)
    store(fn(*args, **kwargs))



import dataclasses
import logging
from functools import partial
from pathlib import Path
from typing import Any, Callable

import hydra_zen
import toolz

log = logging.getLogger(__name__)

import hydra_zen
import toolz
from omegaconf import OmegaConf


def call(fn: Callable):
    return fn()


def aprlify(fn: Callable, inject: dict[str, Callable], params: dict[str, Any]):
    log.debug(f"Params {params}")
    log.debug(f"Injecting {inject} to {fn}")
    fn_cfg = OmegaConf.create(hydra_zen.builds(fn))
    fn_dict = OmegaConf.to_container(fn_cfg)

    for k in sorted(inject.keys(), reverse=False):
        kl = k.split(".")
        log.debug(f"Inserting {k} as {kl}")
        factory_cfg = OmegaConf.create(hydra_zen.just(inject[k]))
        factory_dict = OmegaConf.to_container(factory_cfg)
        fn_dict = toolz.assoc_in(fn_dict, kl, factory_dict)

    log.debug(f"Inserted {fn_dict}")
    log.debug(f"Instantiating...")
    for k in sorted(list(inject.keys()), reverse=True):
        kl = k.split(".")
        log.debug(f"Instantiating and calling {k}, {toolz.get_in(kl, fn_dict)}")
        fn_dict = toolz.update_in(
            fn_dict, kl, toolz.compose_left(hydra_zen.instantiate, call)
        )

    log.debug(f"Injected {fn_dict}")
    return hydra_zen.instantiate(fn_dict)()


def kw2a(fn, **kwargs):
    return fn(*[kwargs[k] for k in sorted(kwargs)])


compose = partial(kw2a, fn=partial(toolz.compose_left))


def make_partial(cfg: type):
    pcfg = dataclasses.make_dataclass(
        cls_name=f"Partial",
        fields=[("_partial_", bool, dataclasses.field(default=True))],
        kw_only=True,
    )
    return dataclasses.make_dataclass(
        cls_name=f"Partial{cfg.__name__}",
        fields=[],
        bases=(pcfg, cfg),
    )


def mkp(path, isdir=None):
    p = Path(path)
    if isdir is None:
        isdir = p.suffix == ""

    if isdir:
        p.mkdir(parents=True, exist_ok=True)
    else:
        p.parent.mkdir(parents=True, exist_ok=True)

    return p


def register_resolvers():
    from omegaconf import OmegaConf

    OmegaConf.register_new_resolver("aprl-mkp", mkp, replace=True)
